/*
 * Copyright (C) 2009 Emanuel Bombasaro, Christian Koch
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with 
 * this program; if not, look at 
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html or write to the Free 
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA  02110-1301, USA.
 * 
 * Contact address: emanuel.bombasaro@cademia.org, christian.koch@cademia.org
 * Homepage       : http://www.rubeam.cademia.org/
 */

package rubeam.model;

import java.awt.geom.Point2D;

public class Node extends ModelObject implements IChangeListener {

    private static final long serialVersionUID = 1L;

    private double x, y;
    private double dphi = Double.NaN; // rad
    private double dy = Double.NaN;
    private double dx = Double.NaN;
    
    public Node(double x, double y, String name) {
        super(name);
        this.x = x;
        this.y = y;
    }

    public Object clone() {
        Node clone = (Node) super.clone(); // Makes a shallow copy
        return clone;
    }

    public void setCoordinates(double x, double y) {
        this.x = x;
        this.y = y;
        _fireWasChanged();
    }

    public void setX(double x) {
        this.x = x;
        _fireWasChanged();
    }
    public void setY(double y) {
        this.y = y;
        _fireWasChanged();
    }
    
    public void setContraints(double fphi, double fy, double fx) {
        this.dphi = fphi;
        this.dy = fy;
        this.dx = fx;
        _fireWasChanged();
    }

    public void setContraintPhi(double fphi) {
        this.dphi = fphi;
        _fireWasChanged();
    }

    public void setContraintX(double fx) {
        this.dx = fx;
        _fireWasChanged();
    }

    public void setContraintY(double fy) {
        this.dy = fy;
        _fireWasChanged();
    }

    public double[] getContraints() {
        return new double[] { dphi, dy, dx };
    }

    public double[] getCoordinates() {
        return new double[] { x, y };
    }

    public Point2D getPoint() {
        return new Point2D.Double(x, y);
    }

    // Listener
    public void wasChanged() {
        _fireWasChanged();
    }
}
