/*
 * Copyright (C) 2009 Emanuel Bombasaro, Christian Koch
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with 
 * this program; if not, look at 
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html or write to the Free 
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA  02110-1301, USA.
 * 
 * Contact address: emanuel.bombasaro@cademia.org, christian.koch@cademia.org
 * Homepage       : http://www.rubeam.cademia.org/
 */

package rubeam.model.loads;

import rubeam.Util;
import rubeam.model.Element;
import Jama.Matrix;

public class ElementBend extends AbstractElementLoad {

    private static final long serialVersionUID = 1L;

    private double phiE;
    private double li;

    public ElementBend(double phiE, double li, String name) {
        super(name);
        this.phiE = phiE;
        this.li = li;
    }

    public double getPhiE() {
        return this.phiE;
    }
    public void setPhiE(double phiE) {
        this.phiE = phiE;
        _fireWasChanged();
    }

    public double getLi() {
        return this.li;
    }
    public void setLi(double li) {
        this.li = li;
        _fireWasChanged();
    }

    public Matrix[] getLoadVectors(double xs) {

        if (xs >= li) {
            Element beam = getElement();
            Matrix Bjs = beam.getBj(xs - li);
            double NII = beam.getNII();

            double[][] _Lji = new double[6][1];
            _Lji[0][0] = 0; // ujL
            _Lji[1][0] = -Bjs.get(0, 1) * phiE; // wjR
            _Lji[2][0] = -Bjs.get(0, 0) * phiE; // phijR
            _Lji[3][0] = Bjs.get(0, 1) * NII * phiE; // MjR
            _Lji[4][0] = 0; // RjR
            _Lji[5][0] = 0; // NjL
            Matrix Lji = new Matrix(_Lji);
            return _buildLoadVectors(Lji);

        }
        else {

            Matrix Lji = new Matrix(6, 1, 0.0);
            return _buildLoadVectors(Lji);
        }

    }

    public String toString() {
        return "PointLoad phiE=" + Util.format(phiE);
    }
}
