/*
 * Copyright (C) 2009 Emanuel Bombasaro, Christian Koch
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with 
 * this program; if not, look at 
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html or write to the Free 
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA  02110-1301, USA.
 * 
 * Contact address: emanuel.bombasaro@cademia.org, christian.koch@cademia.org
 * Homepage       : http://www.rubeam.cademia.org/
 */

package rubeam.model.loads;

import rubeam.Util;
import rubeam.model.Element;
import Jama.Matrix;

public class ElementConLoadAng extends AbstractElementLoad {

    private static final long serialVersionUID = 1L;

    private double F;
    private double angle;
    private double li;

    public ElementConLoadAng(double F, double angle, double li, String name) {
        super(name);
        this.F = F;
        this.angle = angle;
        this.li = li;
    }

    public double getF() {
        return this.F;
    }
    public void setF(double F) {
        this.F = F;
        _fireWasChanged();
    }
    
    public double getAngle() {
        return this.angle;
    }
    public void setAngle(double angle) {
        this.angle = angle;
        _fireWasChanged();
    }
    
    public double getLi() {
        return this.li;
    }
    public void setLi(double li) {
        Element elm = getElement();
        if (elm != null) {
            if (li > elm.getLength())
                li = elm.getLength();
        }
        if (li < 0)
            li = 0;
        this.li = li;
        _fireWasChanged();
    }

    public Matrix[] getLoadVectors(double xs) {
        if (xs >= li) {
            Element element = getElement();
            double E = element.getSection().getE();
            double A = element.getSection().getAi();
            
            Matrix Bjs = element.getBj(xs - li);
            double EI = element.getSection().getEI();          
            
            double Fn = -F * Math.sin(angle);
            double Fd = F * Math.cos(angle);
            
            double[][] _Lji = new double[6][1];
            _Lji[0][0] = (xs - li) / (E * A) * (-Fd); // ujL
            _Lji[1][0] = Bjs.get(0, 3) / EI * Fn; // wjR
            _Lji[2][0] = Bjs.get(0, 2) / EI * Fn; // phijR
            _Lji[3][0] = -Bjs.get(0, 1) * Fn; // MjR
            _Lji[4][0] = -Fn; // RjR
            _Lji[5][0] = -Fd; // NjL
            Matrix Lji = new Matrix(_Lji);
            return _buildLoadVectors(Lji);
        }
        else {
            Matrix Lji = new Matrix(6, 1, 0.0);
            return _buildLoadVectors(Lji);
        }
    }

    public String toString() {
        return "PointForceAng F=" + Util.format(F)  
            + ", alpha=" + Util.format(Math.toDegrees(angle)) 
            + ", Dis. li=" + Util.format(li);
    }
}
