/*
 * Copyright (C) 2009 Emanuel Bombasaro, Christian Koch
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with 
 * this program; if not, look at 
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html or write to the Free 
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA  02110-1301, USA.
 * 
 * Contact address: emanuel.bombasaro@cademia.org, christian.koch@cademia.org
 * Homepage       : http://www.rubeam.cademia.org/
 */

 package rubeam.model.loads;

import rubeam.Util;
import rubeam.model.Element;
import Jama.Matrix;

public class ElementUniDistribLoadSecDir extends AbstractElementLoad {

    private static final long serialVersionUID = 1L;

    private double dn;
    private double li;
    private double lj;

    public ElementUniDistribLoadSecDir(double dn, double li, double lj, String name) {
        super(name);
        this.dn = dn;
        this.li = li;
        this.lj = lj;
    }

    public double getDn() {
        return this.dn;
    }
    public void setDn(double dn) {
        this.dn = dn;
        _fireWasChanged();
    }

    public double getLi() {
        return this.li;
    }
    public void setLi(double li) {
        this.li = li;
        _fireWasChanged();
    }
    
    public double getLj() {
        return this.lj;
    }
    public void setLj(double lj) {
        this.lj = lj;
        _fireWasChanged();
    }
    
    public Matrix[] getLoadVectors(double xs) {

        if (xs <= li) {
            Matrix Lji = new Matrix(6, 1, 0.0);
            return _buildLoadVectors(Lji);
        }
        else {
            Element element = getElement();
            double E = element.getSection().getE();
            double A = element.getSection().getAi();

            double[][] _Lji = new double[6][1];
            _Lji[0][0] = (xs - li) * (xs - li) / (2 * E * A) * dn; // ujL
            _Lji[1][0] = 0; // wjR
            _Lji[2][0] = 0; // phijR
            _Lji[3][0] = 0; // MjR
            _Lji[4][0] = 0; // RjR
            _Lji[5][0] = (xs - li) * dn; // NjL

            if (xs > lj) {

                _Lji[0][0] = _Lji[0][0]
                        - ((xs - lj) * (xs - lj) / (2 * E * A) * dn); // ujL
                _Lji[1][0] = 0; // wjR
                _Lji[2][0] = 0; // phijR
                _Lji[3][0] = 0; // MjR
                _Lji[4][0] = 0; // RjR
                _Lji[5][0] = _Lji[5][0] - ((xs - lj) * dn); // NjL
            }

            Matrix Lji = new Matrix(_Lji);
            return _buildLoadVectors(Lji);
        }
    }

    public String toString() {
        return "UniDistribLoad dn=" + Util.format(dn);
    }
}
