/*
 * Copyright (C) 2009 Emanuel Bombasaro, Christian Koch
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with 
 * this program; if not, look at 
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html or write to the Free 
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA  02110-1301, USA.
 * 
 * Contact address: emanuel.bombasaro@cademia.org, christian.koch@cademia.org
 * Homepage       : http://www.rubeam.cademia.org/
 */

package rubeam.model.loads;

import rubeam.Util;
import Jama.Matrix;

public class ElementUniPreStress extends AbstractElementLoad {

    private static final long serialVersionUID = 1L;

    private double eE;

    // eE = T * alpha_T so we can simulate pretension of the element
    // with NE = EA * eE

    public ElementUniPreStress(double eE, String name) {
        super(name);
        this.eE = eE;
    }

    public double getEE() {
        return this.eE;
    }
    public void setEE(double eE) {
        this.eE = eE;
        _fireWasChanged();
    }

    public Matrix[] getLoadVectors(double xs) {

        double[][] _Lji = new double[6][1];
        _Lji[0][0] = xs * eE; // ujL
        _Lji[1][0] = 0; // wjR
        _Lji[2][0] = 0; // phijR
        _Lji[3][0] = 0; // MjR
        _Lji[4][0] = 0; // QjR
        _Lji[5][0] = 0; // NjL
        Matrix Lji = new Matrix(_Lji);
        return _buildLoadVectors(Lji);

    }

    public String toString() {
        return "PreStress eE=" + Util.format(eE);
    }
}
