/*
 * Copyright (C) 2009 Emanuel Bombasaro, Christian Koch
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with 
 * this program; if not, look at 
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html or write to the Free 
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA  02110-1301, USA.
 * 
 * Contact address: emanuel.bombasaro@cademia.org, christian.koch@cademia.org
 * Homepage       : http://www.rubeam.cademia.org/
 */

package rubeam.model.sections;


public class LinearIProfileSection extends AbstractSection {

    private static final long serialVersionUID = 1L;
    
    private double hi, hk, s, b, t;
    
    public LinearIProfileSection(double hi, double hk, double b, double s, double t, double E, String name) {
        super(name, E);
        this.hi = hi;
        this.hk = hk;
        this.s = s;
        this.b = b;
        this.t = t;
    }
    
    public double getAi() {
        return (hi-t) * s + 2 * b * t;
    }
    public double getAk() {
        return (hk-t) * s + 2 * b * t;
    }
    public double getIi() {
        return (hi-t) * (hi-t) * ((hi-t) * s / 12 + b * t/ 2);
    }

    public double getHi() {
        return hi;
    }
    public void setHi(double hi) {
        this.hi = hi;
        _fireWasChanged();
    }

    public double getHk() {
        return hk;
    }
    public void setHk(double hk) {
        this.hk = hk;
        _fireWasChanged();
    }

    public double getS() {
        return s;
    }
    public void setS(double s) {
        this.s = s;
        _fireWasChanged();
    }
    public double getB() {
        return b;
    }
    public void setB(double b) {
        this.b = b;
        _fireWasChanged();
    }
    public double getT() {
        return t;
    }
    public void setT(double t) {
        this.t = t;
        _fireWasChanged();
    }

    public String toString() {
        return "I-Profil Section";
    }
}
