/*
 * Copyright (C) 2009 Emanuel Bombasaro, Christian Koch
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with 
 * this program; if not, look at 
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html or write to the Free 
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA  02110-1301, USA.
 * 
 * Contact address: emanuel.bombasaro@cademia.org, christian.koch@cademia.org
 * Homepage       : http://www.rubeam.cademia.org/
 */

package rubeam.cmds;

import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;

import rubeam.RuBeam;
import cib.cad.kernel.Kernel;
import cib.util.cmd.CmdAbortedException;
import cib.util.cmd.CmdAdapter;

/** 
 * Exports ruBeam macro file.
 * 
 */
public class ExportMacro extends CmdAdapter {
    
    public void doCmd(Object context) throws CmdAbortedException {
        Kernel krnl = (Kernel)context;
        final String EXT = ImportMacro.MACRO_EXTENSION;
        
        // Show file dialog
        File file = krnl.readFile(new Kernel.FileDialogConfiguration() {
            public boolean accept(File f) {
                return f.isDirectory() ||
                f.getName().toLowerCase().endsWith("." + EXT);
            }
            public String[] getExtensions() {
                return new String[] {EXT};
            }
            public String getPrompt() {
                return "Export ruBeam macro";
            }
            public boolean isOpenDialog() {
                return false;   // Is SAVE dialog
            }
            public File getCurrentFile() {
                return new File("macro." + EXT);
            }
            public String getFileFormatDescription() {
                return "ruBeam-files (*." + EXT + ")";
            }
        });
        
        if (file.exists()) {
            System.out.println("File already exists ...");
        }
        else {
            System.out.println("File newly created ...");
            try {
                file.createNewFile();
            }
            catch (IOException e) {
                System.err.println(e.getMessage());
                return;
            }
        }
              
        if (! file.canWrite()) {
            System.err.println("Can't write to " + file.getAbsolutePath());
            return;
        }
        
        try {
            PrintWriter pw = new PrintWriter(file);
            pw.write(RuBeam.getStructure().exportToString());
            pw.flush();
            pw.close();
        }
        catch(IOException e) {
            System.err.println(e);
            return;
        }
        
        System.out.println("Database " + file.getAbsolutePath() +
            " successfully stored");
    }
    public String toString() {
        return "Macro file successfully exported.";
    }
}
