/*
 * Copyright (C) 2009 Emanuel Bombasaro, Christian Koch
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with 
 * this program; if not, look at 
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html or write to the Free 
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA  02110-1301, USA.
 * 
 * Contact address: emanuel.bombasaro@cademia.org, christian.koch@cademia.org
 * Homepage       : http://www.rubeam.cademia.org/
 */

package rubeam.cmds.elm;

import java.awt.Rectangle;
import java.awt.Window;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.util.Iterator;
import java.util.Set;

import rubeam.RuBeam;
import rubeam.cmps.ComponentElement;
import rubeam.cmps.ComponentNode;
import rubeam.model.Element;
import rubeam.model.Node;
import rubeam.model.Structure;
import cib.cad.db.Database;
import cib.cad.db.att.AttributeManager;
import cib.cad.db.att.Attributes;
import cib.cad.db.comp.Component;
import cib.cad.kernel.GeometryPanel;
import cib.cad.kernel.Kernel;
import cib.cad.kernel.UserInterface;
import cib.cad.kernel.cmds.util.LineRubberBand;
import cib.util.CoordSpace;
import cib.util.NameSpace;
import cib.util.cmd.CmdAbortedException;
import cib.util.cmd.CmdAdapter;
import cib.util.coll.ClassFilter;
import cib.util.coll.CollUtilities;
import cib.util.view.ViewUtilities;

/**
 * @author Christian Koch
 */
public class AddElement extends CmdAdapter {

    private ComponentElement m_beam = null;
    private ComponentNode m_node1;
    private ComponentNode m_node2;
    private Structure m_structure = null;
    private boolean m_addedNode1 = false;
    private boolean m_addedNode2 = false;

    public void doCmd(Object context) throws CmdAbortedException {
        Kernel krnl = (Kernel) context;
        Database db = krnl.getDatabase();
        CoordSpace cs = CoordSpace.getCoordSpace();
        NameSpace ns = db.getNameSpace();
        Set<Component> compSet = db.getComponentSet();
        AttributeManager atm = db.getAttributeManager();
        Attributes attr = atm.getDefaultAttributes();
        m_structure = RuBeam.getStructure();
        ViewUtilities.requestFocusAsynchronously(
                (Window)krnl.getUserInterface());
        
        try {
            // first
            if (krnl.hasArgument()) {
                String name1 = krnl.readString("Enter name1");
                Iterator<Component> eit = CollUtilities.filterableIterator(
                        compSet.iterator(), new ClassFilter(
                                ComponentNode.class));
                while (eit.hasNext()) {
                    ComponentNode cNode = (ComponentNode) eit.next();
                    Node node = cNode.getNode();
                    if (node.getName().equalsIgnoreCase(name1)) {
                        m_node1 = cNode;
                        break;
                    }
                }
            }
            if (m_node1 == null) {
                Point2D pt = krnl.readPoint("Set first node");
                GeometryPanel gp = krnl.getUserInterface()
                        .geometryPanelIterator().next();
                AffineTransform m2v = gp.getProjection().getModelToView();
                Point2D vp = m2v.transform(pt, null);

                Rectangle rectangle = new Rectangle();
                rectangle.setFrameFromDiagonal(vp.getX()
                        - UserInterface.PICK_BOX_SIZE / 2, vp.getY()
                        - UserInterface.PICK_BOX_SIZE / 2, vp.getX()
                        + UserInterface.PICK_BOX_SIZE / 2, vp.getY()
                        + UserInterface.PICK_BOX_SIZE / 2);
                Iterator<Component> it = gp.objectIterator(rectangle, true);
                boolean found = false;
                if (it.hasNext() && !found) { // Use existing node
                    Object o = it.next();
                    if (o instanceof ComponentNode) {
                        m_node1 = (ComponentNode) o;
                        found = true;
                    }
                }
                if (m_node1 == null) { // create new node
                    Point2D ptM = cs.getUserCrdTransform().transform(pt, null);
                    Node node = new Node(ptM.getX(), ptM.getY(), ns
                            .generateName(RuBeam.NODE_PREFIX));
                    m_node1 = new ComponentNode(node);
                    m_structure.addNode(node);
                    compSet.add(m_node1);
                    m_addedNode1 = true;
                    krnl.getUserInterface().updateUserInterface();
                }
            }

            LineRubberBand rb = null;
            try {
                // second
                if (krnl.hasArgument()) {
                    String name2 = krnl.readString("Enter name2");
                    Iterator<Component> eit = CollUtilities.filterableIterator(
                            compSet.iterator(), new ClassFilter(
                                    ComponentNode.class));
                    while (eit.hasNext()) {
                        ComponentNode cNode = (ComponentNode) eit.next();
                        Node node = cNode.getNode();
                        if (node.getName().equalsIgnoreCase(name2)) {
                            m_node2 = cNode;
                            break;
                        }
                    }
                }
                if (m_node2 == null) {

                    // Create rubber band
                    rb = new LineRubberBand(krnl, m_node1.getCenter());

                    Point2D pt = krnl.readPoint("Set second node");
                    GeometryPanel gp = krnl.getUserInterface()
                            .geometryPanelIterator().next();
                    AffineTransform m2v = gp.getProjection().getModelToView();
                    Point2D vp = m2v.transform(pt, null);
                    Rectangle rectangle = new Rectangle();
                    rectangle.setFrameFromDiagonal(vp.getX()
                            - UserInterface.PICK_BOX_SIZE / 2, vp.getY()
                            - UserInterface.PICK_BOX_SIZE / 2, vp.getX()
                            + UserInterface.PICK_BOX_SIZE / 2, vp.getY()
                            + UserInterface.PICK_BOX_SIZE / 2);
                    Iterator<Component> it = gp.objectIterator(rectangle, true);
                    boolean found = false;
                    if (it.hasNext() && !found) { // Use existing node
                        Object o = it.next();
                        if (o instanceof ComponentNode) {
                            m_node2 = (ComponentNode) o;
                            found = true;
                        }
                    }
                    if (m_node2 == null) { // create new node
                        Point2D ptM = cs.getUserCrdTransform().transform(pt, null);
                        Node node = new Node(ptM.getX(), ptM.getY(), ns
                                .generateName(RuBeam.NODE_PREFIX));
                        m_node2 = new ComponentNode(node);
                        m_structure.addNode(node);
                        compSet.add(m_node2);
                        m_addedNode2 = true;
                        krnl.getUserInterface().updateUserInterface();
                    }
                }
            } finally {
                if (rb != null)
                    rb.close();
            }
        } catch (CmdAbortedException e) {
            if (m_addedNode1) {
                m_structure.removeNode(m_node1.getNode());
                compSet.remove(m_node1);
            }
            throw e;
        }
        String name;
        if (krnl.hasArgument()) {
            name = krnl.readString("Enter name");
        }  
        else {
            name = ns.generateName(RuBeam.ELEMENT_PREFIX);
        }
        Element beam = new Element(m_node1.getNode(), m_node2.getNode(), name);
        if (m_structure.containsElement(beam)/* ns.containsName(m_edge.getName()) */) {
            krnl.readOK("Edge from " + beam.getNodeI().getName() + " to "
                    + beam.getNodeK().getName() + " already exists.",
                    Kernel.DEFAULT_OK);
            return;
        }
        m_beam = new ComponentElement(beam);
        m_beam.setAttributes(attr);

        m_structure.addElement(beam);
        compSet.add(m_beam);

        System.out.println(this);
    }

    public void undoCmd(Object context) {
        Kernel krnl = (Kernel) context;
        Database db = krnl.getDatabase();
        Set<Component> cmpSet = db.getComponentSet();
        if (m_addedNode1) {
            m_structure.removeNode(m_node1.getNode());
            cmpSet.remove(m_node1);
        }
        if (m_addedNode2) {
            m_structure.removeNode(m_node2.getNode());
            cmpSet.remove(m_node2);
        }
        m_structure.removeElement(m_beam.getElement());
        cmpSet.remove(m_beam);
    }

    public void redoCmd(Object context) {
        Kernel krnl = (Kernel) context;
        Database db = krnl.getDatabase();
        Set<Component> cmpSet = db.getComponentSet();
        if (m_addedNode1) {
            m_structure.addNode(m_node1.getNode());
            cmpSet.add(m_node1);
        }
        if (m_addedNode2) {
            m_structure.addNode(m_node2.getNode());
            cmpSet.add(m_node2);
        }
        m_structure.addElement(m_beam.getElement());
        cmpSet.add(m_beam);
    }

    public boolean changesState() {
        return true;
    }

    public boolean isUndoable() {
        return true;
    }

    public String toString() {
        return "Added element at (" + m_beam.getElement().getNodeI().getName() + ", "
                + m_beam.getElement().getNodeK().getName() + ").";
    }
}
