/*
 * Copyright (C) 2009 Emanuel Bombasaro, Christian Koch
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with 
 * this program; if not, look at 
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html or write to the Free 
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA  02110-1301, USA.
 * 
 * Contact address: emanuel.bombasaro@cademia.org, christian.koch@cademia.org
 * Homepage       : http://www.rubeam.cademia.org/
 */

package rubeam.cmds.elm;

import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import rubeam.RuBeam;
import rubeam.cmps.ComponentElement;
import rubeam.cmps.ComponentElementLoad;
import rubeam.model.Element;
import rubeam.model.Node;
import rubeam.model.loads.AbstractElementLoad;
import cib.cad.db.comp.Component;
import cib.cad.kernel.Kernel;
import cib.util.cmd.Cmd;
import cib.util.cmd.CmdAbortedException;
import cib.util.coll.ClassFilter;
import cib.util.coll.CollUtilities;
import cib.util.coll.ObservableSet;

public class SetElementNodeK implements Cmd {
    
    private Node m_node = null; // undo data
    private ComponentElement m_cElement = null;
    private Set<ComponentElementLoad> m_cLoads = null;
    
    public void doCmd(Object context) throws CmdAbortedException {
        Kernel krnl = (Kernel) context;
        ObservableSet<Component> cmpSet = krnl.getDatabase().getComponentSet();
        
        String name = krnl.readString("Enter element name");
        // find node
        Iterator<Component> nit = CollUtilities.filterableIterator(
                cmpSet.iterator(), new ClassFilter(
                        ComponentElement.class));
        Element element = null;
        while (nit.hasNext()) {
            ComponentElement cElement = (ComponentElement) nit.next();
            Element e = cElement.getElement();
            if (name.equals(e.getName())) {
                m_cElement = cElement;
                element = e;
                break;
            }
        }
        if (element != null) {
            String newName = krnl.readString("Enter name of node k");
            List<Node> nodeList = RuBeam.getStructure().readNodes();
            Node nk = null;
            for (Node node : nodeList) {
                if (newName.equals(node.getName())) {
                    nk = node;
                    break;
                }
            }
            if (nk != null && nk != element.getNodeI()) {
                m_node = element.getNodeK();
                element.setNodeK(nk);
                // get assign loads
                List<AbstractElementLoad> loads = 
                    RuBeam.getStructure().getElementLoads(element);
                if (loads != null && !loads.isEmpty()) {
                    Iterator<Component> lit = CollUtilities.filterableIterator(
                            cmpSet.iterator(), new ClassFilter(
                                    ComponentElementLoad.class));
                    while (lit.hasNext()) {
                        ComponentElementLoad cl = (ComponentElementLoad) lit.next();
                        if (loads.contains(cl.getLoad())) {
                            if (m_cLoads == null)
                                m_cLoads = new HashSet<ComponentElementLoad>();
                            cmpSet.wasChanged(cl); 
                            m_cLoads.add(cl);
                        }
                    }
                }
                cmpSet.wasChanged(m_cElement);
                System.out.println(this);
                return;
            }
        }
        System.out.println("Element/ node not found or ni == nk.");
        
    }

    public void undoCmd(Object context) {
        Kernel krnl = (Kernel) context;
        ObservableSet<Component> cmpSet = krnl.getDatabase().getComponentSet();
        Node tmp = m_cElement.getElement().getNodeK();
        m_cElement.getElement().setNodeK(m_node);
        m_node = tmp;
        if (m_cLoads != null) {
            for (ComponentElementLoad cl : m_cLoads)
                cmpSet.wasChanged(cl);
        }
        cmpSet.wasChanged(m_cElement);
            
    }

    public void redoCmd(Object context) {
        undoCmd(context);
    }
    public boolean isUndoable() {
        return true;
    }
    public boolean changesState() {
        return (m_node != null);
    }
    public String toString() {
        return "Set node k to '"
            + m_cElement.getElement().getNodeK().getName() + "'" 
            + " for element '" + m_cElement.getElement().getName() + "'.";
    }
}
