/*
 * Copyright (C) 2009 Emanuel Bombasaro, Christian Koch
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with 
 * this program; if not, look at 
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html or write to the Free 
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA  02110-1301, USA.
 * 
 * Contact address: emanuel.bombasaro@cademia.org, christian.koch@cademia.org
 * Homepage       : http://www.rubeam.cademia.org/
 */

package rubeam.cmds.elmload;

import java.util.Iterator;

import rubeam.cmps.ComponentElementConLoadAng;
import rubeam.model.loads.ElementConLoadAng;
import cib.cad.db.comp.Component;
import cib.cad.kernel.Kernel;
import cib.util.cmd.Cmd;
import cib.util.cmd.CmdAbortedException;
import cib.util.coll.ClassFilter;
import cib.util.coll.CollUtilities;
import cib.util.coll.ObservableSet;

public class SetConLoadAngLiAbs implements Cmd {
    
    private double m_li; // undo data 
    private ComponentElementConLoadAng m_cLoad = null;
    
    public void doCmd(Object context) throws CmdAbortedException {
        Kernel krnl = (Kernel) context;
        ObservableSet<Component> cmpSet = krnl.getDatabase().getComponentSet();
        
        String name = krnl.readString("Enter element load name");
        // find node
        Iterator<Component> nit = CollUtilities.filterableIterator(
                cmpSet.iterator(), new ClassFilter(
                        ComponentElementConLoadAng.class));
        ElementConLoadAng load = null;
        while (nit.hasNext()) {
            ComponentElementConLoadAng cLoad = (ComponentElementConLoadAng) nit.next();
            ElementConLoadAng l = (ElementConLoadAng)cLoad.getLoad();
            if (name.equals(l.getName())) {
                m_cLoad = cLoad;
                m_li = l.getLi();
                load = l;
                break;
            }
        }
        if (load != null) {
            double newValue = krnl.readDouble("Enter new value");
            load.setLi(newValue);
            cmpSet.wasChanged(m_cLoad);
            System.out.println(this);
        }
        else {
            System.out.println("Element load '" + name + "' not found.");
        }
    }

    public void undoCmd(Object context) {
        Kernel krnl = (Kernel) context;
        ObservableSet<Component> cmpSet = krnl.getDatabase().getComponentSet();
        ElementConLoadAng load = (ElementConLoadAng)m_cLoad.getLoad();
        double tmp = load.getLi();
        load.setLi(m_li);
        m_li = tmp;
        cmpSet.wasChanged(m_cLoad);
    }

    public void redoCmd(Object context) {
        undoCmd(context);
    }
    public boolean isUndoable() {
        return true;
    }
    public boolean changesState() {
        return (m_cLoad != null);
    }
    public String toString() {
        return "Set li of element load'" + m_cLoad.getLoad().getName() 
            + "' to " + ((ElementConLoadAng)m_cLoad.getLoad()).getLi(); 
    }
}
