/*
 * Copyright (C) 2009 Emanuel Bombasaro, Christian Koch
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with 
 * this program; if not, look at 
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html or write to the Free 
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA  02110-1301, USA.
 * 
 * Contact address: emanuel.bombasaro@cademia.org, christian.koch@cademia.org
 * Homepage       : http://www.rubeam.cademia.org/
 */

package rubeam.cmds.node;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import rubeam.cmps.ComponentNode;
import rubeam.model.Node;
import cib.cad.db.comp.Component;
import cib.cad.kernel.Kernel;
import cib.cad.kernel.cmds.Remove;
import cib.util.cmd.CmdAbortedException;
import cib.util.coll.ClassFilter;
import cib.util.coll.CollUtilities;
import cib.util.coll.ObservableSet;

public class RemoveNode extends Remove {
    
    private ComponentNode m_cNode = null;
    private Set<Component> m_selSet = null;
    
    public void doCmd(Object context) throws CmdAbortedException {
        Kernel krnl = (Kernel) context;
        ObservableSet<Component> cmpSet = krnl.getDatabase().getComponentSet();
        Set<Component> selSet = krnl.getDatabase().getSelectSet();
        
        String name = krnl.readString("Enter node name");
        
        // find node
        Iterator<Component> nit = CollUtilities.filterableIterator(
                cmpSet.iterator(), new ClassFilter(
                        ComponentNode.class));
        Node node = null;
        while (nit.hasNext()) {
            ComponentNode cNode = (ComponentNode) nit.next();
            Node n = cNode.getNode();
            if (name.equals(n.getName())) {
                m_cNode = cNode;
                node = n;
                break;
            }
        }
        if (m_cNode != null && node != null) {
            // save selSet
            m_selSet = new HashSet<Component>();
            m_selSet.addAll(selSet);
            selSet.clear();
            selSet.add(m_cNode);
            
            super.doCmd(context);
            
            // restore selSet
            selSet.addAll(m_selSet);
        }
        else {
            System.out.println("Node '" + name + "' not found.");
        }
    }

    public void undoCmd(Object context) {
        Kernel krnl = (Kernel) context;
        Set<Component> selSet = krnl.getDatabase().getSelectSet();
        super.undoCmd(context);
        selSet.remove(m_cNode);
    }

    public void redoCmd(Object context) {
        Kernel krnl = (Kernel) context;
        Set<Component> selSet = krnl.getDatabase().getSelectSet();
        super.redoCmd(context);
        selSet.addAll(m_selSet);
    }
    public boolean isUndoable() {
        return super.isUndoable();
    }
    public boolean changesState() {
        return super.changesState();
    }
    public String toString() {
        return "Node '" + m_cNode.getNode().getName() + "' removed.";
    }
}
