/*
 * Copyright (C) 2009 Emanuel Bombasaro, Christian Koch
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with 
 * this program; if not, look at 
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html or write to the Free 
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA  02110-1301, USA.
 * 
 * Contact address: emanuel.bombasaro@cademia.org, christian.koch@cademia.org
 * Homepage       : http://www.rubeam.cademia.org/
 */

package rubeam.cmds.sec;

import java.util.HashSet;
import java.util.Set;

import rubeam.RuBeam;
import rubeam.model.Element;
import rubeam.model.Structure;
import rubeam.model.sections.AbstractSection;
import cib.cad.kernel.Kernel;
import cib.util.cmd.Cmd;
import cib.util.cmd.CmdAbortedException;

public class RemoveSection implements Cmd {
    
    private AbstractSection m_section = null;
    private Set<Element> m_elms = null;
    
    public void doCmd(Object context) throws CmdAbortedException {
        Kernel krnl = (Kernel) context;
        Structure struct = RuBeam.getStructure();
        
        String name = krnl.readString("Enter section name");
        System.out.println("name=" + name);
        // find section
        for (AbstractSection sec : struct.readSections()) {
            if (sec.getName().equals(name) 
                    && sec != AbstractSection.getDefaultSection()) {
                m_section = sec;
                break;
            }
        }
        if (m_section != null) {
            // save old section assignments
            m_elms = new HashSet<Element>();
            for (Element elm : struct.readElements()) {
                if (elm.getSection() == m_section) {
                    m_elms.add(elm);
                }
            }
            struct.removeSection(m_section);
        }
    }

    public void undoCmd(Object context) {
        Structure struct = RuBeam.getStructure();
        struct.addSection(m_section);
        for (Element elm : m_elms) {
            struct.putSection(elm, m_section);
        }
    }

    public void redoCmd(Object context) {
        RuBeam.getStructure().removeSection(m_section);
    }
    public boolean isUndoable() {
        return true;
    }
    public boolean changesState() {
        return (m_section != null);
    }
    public String toString() {
        return "Section '" + m_section.getName() + "' removed.";
    }
}
