/*
 * Copyright (C) 2009 Emanuel Bombasaro, Christian Koch
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with 
 * this program; if not, look at 
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html or write to the Free 
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA  02110-1301, USA.
 * 
 * Contact address: emanuel.bombasaro@cademia.org, christian.koch@cademia.org
 * Homepage       : http://www.rubeam.cademia.org/
 */

package rubeam.cmds.sec;

import rubeam.RuBeam;
import rubeam.model.sections.AbstractSection;
import cib.cad.kernel.Kernel;
import cib.util.cmd.Cmd;
import cib.util.cmd.CmdAbortedException;

public class SetSectionIi implements Cmd {
    
    private double m_Ii; // undo data
    private AbstractSection m_section = null;
    
    public void doCmd(Object context) throws CmdAbortedException {
        Kernel krnl = (Kernel) context;
        
        String m_name = krnl.readString("Enter section name");
        // find section
        for (AbstractSection sec : RuBeam.getStructure().readSections()) {
            if (sec.getName().equals(m_name)) {
                try {
                    m_Ii = sec.getIi();
                    double newIi = krnl.readDouble("Enter new Ii");
                    sec.setIi(newIi);
                    m_section = sec;
                }
                catch (UnsupportedOperationException e) {
                    System.err.println(e.getMessage());
                    return;
                }
                System.out.println(this);
                return;
            }
        }
        System.out.println("Section '" + m_name + "' not found.");
    }

    public void undoCmd(Object context) {
        double tmp = m_section.getIi();
        m_section.setIi(m_Ii);
        m_Ii = tmp;
    }

    public void redoCmd(Object context) {
        undoCmd(context);
    }
    public boolean isUndoable() {
        return true;
    }
    public boolean changesState() {
        return (m_section != null);
    }
    public String toString() {
        return "Set Ii of section '" + m_section.getName() + "' to "
            + m_section.getIi() + ".";
    }
}
