/*
 * Copyright (C) 2009 Emanuel Bombasaro, Christian Koch
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with 
 * this program; if not, look at 
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html or write to the Free 
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA  02110-1301, USA.
 * 
 * Contact address: emanuel.bombasaro@cademia.org, christian.koch@cademia.org
 * Homepage       : http://www.rubeam.cademia.org/
 */

package rubeam.gui;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;

import javax.swing.AbstractCellEditor;
import javax.swing.JComboBox;
import javax.swing.JTable;
import javax.swing.table.TableCellEditor;

import rubeam.RuBeam;
import rubeam.model.Node;

public class NodeListCellEditor extends AbstractCellEditor implements TableCellEditor {

    private static final long serialVersionUID = 1L;
    private JComboBox comboBox = new JComboBox();
    private int index = 0;
    private ActionListener listener;
    
    public NodeListCellEditor() {
        listener = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                index = comboBox.getSelectedIndex();
                fireEditingStopped();
            } 
         };
         comboBox.addActionListener(listener);
    }
    
    public Component getTableCellEditorComponent(JTable table, Object value,
        boolean isSelected, int row, int column) {
        List<Node> nodeList = RuBeam.getStructure().readNodes();
        comboBox.removeActionListener(listener);
        comboBox.removeAllItems();
        for (Node node : nodeList) {
            String name = node.getName();
            comboBox.addItem(name);
            if (name.equals(value.toString()))
                comboBox.setSelectedItem(name);
            
        }
        comboBox.addActionListener(listener);
        return comboBox;
    }

    public Object getCellEditorValue() {
        return comboBox.getItemAt(index);
    }
    
}
