/*
 * Copyright (C) 2009 Emanuel Bombasaro, Christian Koch
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with 
 * this program; if not, look at 
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html or write to the Free 
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA  02110-1301, USA.
 * 
 * Contact address: emanuel.bombasaro@cademia.org, christian.koch@cademia.org
 * Homepage       : http://www.rubeam.cademia.org/
 */

package rubeam.gui;

import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.util.Locale;

import javax.swing.SwingConstants;
import javax.swing.table.DefaultTableCellRenderer;

public class NumberCellRenderer extends DefaultTableCellRenderer {
    private static final long serialVersionUID = 1L;
    
    private DecimalFormat formatter = new DecimalFormat();
 
    public NumberCellRenderer() {
        super();
        this.formatter.setDecimalFormatSymbols(new DecimalFormatSymbols(Locale.ENGLISH));
        setHorizontalAlignment( SwingConstants.RIGHT );
    }
 
    public void setValue(Object value) {
        if ((value != null) && (value instanceof Number)) {
            Number num = (Number) value;
            if (Double.isNaN(num.doubleValue())) {
                value = "";
            }
            else {           
                double val = num.doubleValue();
                if (val > -1 && val < 1) {
                    if (val != 0.0) {
                        if ((val > 0.0 && val < 0.001)
                                || (val < 0.0 && val > -0.001)) {
                            formatter.applyLocalizedPattern("0.00###E0");
                        }
                        else {
                            formatter.applyLocalizedPattern("0.#####");
                        }
                    }
                    else {
                        formatter.applyLocalizedPattern("0.#####");
                    }
                }
                else if ( val > 9999 || val < -9999) {
                    formatter.applyLocalizedPattern("##0.#####E0");
                }
                else {
                    formatter.applyLocalizedPattern("####.#####");
                }
                value = formatter.format(val);
            }
        } 
        super.setValue(value);
    }
}
