/* 
 * Copyright (C) 2002-2010 Berthold Firmenich
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, look at "http://www.gnu.org/copyleft/gpl.html" or
 * write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 * 
 * Contact address: berthold.firmenich@cademia.org
 * Homepage       : http://www.cademia.org/
 */

package cib.util;


/** 
 * Debugging support
 * 
 * @author Berthold Firmenich
 * @version $Revision: 1.5 $, $Author: cvs-cad-firmenic $
 * @version $Date: 2010/01/13 15:05:11 $
 */
public class Dbg {

    private static boolean s_on = false;

    private Dbg() { // Constructor disabled
    }

   /** 
    * Query debug message output status. On startup debug message output is
    * disabled.
    * @return status
    */
    public static boolean on() {
        return s_on;
    }

   /** 
    * Set debug message output status on
    * @param on flag
    */
    public static void set(boolean on) {
        s_on = on;
    }

   /** 
    * Prints the stack trace of a Throwable
    * @param e the Throwable
    */
    public static void println(Throwable e) {
        if (s_on)
            e.printStackTrace();
    }
   /** 
    * Prints a debug message with line termination
    * @param o an object (intended to be the caller)
    * @param s the String to be printed
    */
    public static void println(Object o, String s) {
        if (s_on)
            System.err.println(o.getClass().getName() + " " + s);
    }
   /** 
    * Prints a debug message with line termination
    * @param s the String to be printed
    */
    public static void println(String s) {
        if (s_on)
            System.err.println(s);
    }

   /** 
    * Prints a debug message with
    * @param o an object (intended to be the caller)
    * @param s the String to be printed
    */
    public static void print(Object o, String s) {
        if (s_on)
            System.err.print(o.getClass().getName() + " " + s);
    }
   /** 
    * Prints a debug message with
    * @param s the String to be printed
    */
    public static void print(String s) {
        if (s_on)
            System.err.print(s);
    }
}