/* 
 * Copyright (C) 2008-2010 Berthold Firmenich
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, look at "http://www.gnu.org/copyleft/gpl.html" or
 * write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 * 
 * Contact address: berthold.firmenich@cademia.org
 * Homepage       : http://www.cademia.org/
 */

package cib.util.binding;

import java.io.Serializable;
import java.util.HashSet;
import java.util.Set;

/** 
 * A BindingAdapter represents the implementation of a Binding.
 * 
 * @param <E> the type of the bound object and the binding objects
 * 
 * @author Albertus Olivier, Berthold Firmenich
 * @version $Revision: 1.5 $, $Author: cvs-cad-firmenic $
 * @version $Date: 2010/01/13 15:05:11 $
 */
public class BindingAdapter<E> extends Binding<E> implements Serializable {
	private static final long serialVersionUID = 1L;

	private E m_dst = null;
	private HashSet<E> m_src = new HashSet<E>();
	
    /**
     * Clones this BindingAdapter. The clone will have the same bound and
     * binding objects as this. Because a bound object can only have one Binding
     * this object state is only acceptable as long as the clone is not added to
     * the {@link BindingModel}.
     * 
     * @return the cloned object
     */
	public Object clone() {
	    // Shallow copy
	    BindingAdapter<E> clone = (BindingAdapter<E>)super.clone();

        // Extend to deep copy
        clone.m_src = (HashSet<E>)m_src.clone();
        
        return clone;
	}

    /* (non-Javadoc)
     * @see cib.util.binding.Binding#getBoundObject()
     */
    public E getBoundObject() {
        return m_dst;
    }
    
    /* (non-Javadoc)
     * @see cib.util.binding.Binding#_setBoundObject(java.lang.Object)
     */
    protected void _setBoundObject(E obj) {
        m_dst = obj;
    }
    
    /* (non-Javadoc)
     * @see cib.util.binding.Binding#_addBindingObject(java.lang.Object)
     */
    protected void _addBindingObject(E obj) {
        m_src.add(obj);
    }
    
    /* (non-Javadoc)
     * @see cib.util.binding.Binding#removeBindingObject(java.lang.Object)
     */
    public void removeBindingObject(E obj) {
        m_src.remove(obj);
    }
	/* (non-Javadoc)
	 * @see cib.util.binding.Binding#getBindingObjects()
	 */
	public Set<E> getBindingObjects() {
        return m_src;
	}
	
	/* (non-Javadoc)
	 * @see cib.util.binding.Binding#update(cib.util.binding.BindingContext)
	 */
	public void update(BindingContext<E> context) {
		System.out.println();
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
        return "{" + m_src + " -> " + m_dst + "}";
	}
}
