/* 
 * Copyright (C) 2008-2010 Berthold Firmenich
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, look at "http://www.gnu.org/copyleft/gpl.html" or
 * write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 * 
 * Contact address: berthold.firmenich@cademia.org
 * Homepage       : http://www.cademia.org/
 */

package cib.util.binding;

import java.io.Serializable;
import java.util.Map;

/** 
 * Represents a BindingModel. The BindingModel maps Bindings to Objects. This
 * ensures that each bound object can be assigned at most one single Binding.
 * <p>
 * The BindingModel ensures that the BindingGraph is free of cycles.
 * 
 * @param <E> the type of the bound object and the binding objects
 * 
 * @author Albertus Olivier, Berthold Firmenich
 * @version $Revision: 1.3 $, $Author: cvs-cad-firmenic $
 * @version $Date: 2010/01/13 15:05:11 $
 */
public interface BindingModel<E> extends Map<E,Binding<E>>, Serializable {
    /**
     * Adds a binding object to a Binding if the binding graph remains acyclic.
     * 
     * @param binding the Binding
     * @param obj the binding object
     * @exception IllegalArgumentException in case of a cycle.
     */
    public void addBindingObject(Binding<E> binding, E obj);
}
