package cib.util.binding;

import java.util.HashSet;
import java.util.Set;

public class BindingTest {

    public static void main(String[] args) {
        /*
         *   Expression: a + b * c
         * 
         * 
         *   Syntax tree with bindings:
         * 
         *       +
         *      / \     Binding bPlus
         *     *   a
         *    / \       Binding bTimes
         *   b   c
         * 
         * 
         */

        // Class Node represents a double number 
        class Node {
            String name = "Unnamed";
            double x = 0;
            Node(String name, double x) {
                this.name = name;
                this.x = x;
            }
            public String toString() {
                return this.name + "(" + Double.toString(x) + ")";
            }
        }
        
        // Binding space
        BindingModel<Node> bindingModel = new DefaultBindingModel<Node>();
        
        // The Node instances
        Node a = new Node("a", 1);
        Node b = new Node("b", 2);
        Node c = new Node("c", 3);
        Node times = new Node("times", 0);
        Node plus = new Node("plus", 0);

        // The binding of the '*' Node
        Binding<Node> bTimes = new BindingAdapter<Node>() {
            private static final long serialVersionUID = 1L;
            public void update(BindingContext<Node> context) {
                System.out.println("bTimes:update()");
                Node bound = getBoundObject();
                Set<Node> bindingObjects = getBindingObjects();
                bound.x = 1;
                for (Node node : bindingObjects) {
                    bound.x *= node.x;
                }
            }
            public Object clone() {
                return super.clone();
            }
        };
        bindingModel.addBindingObject(bTimes, b);
        bindingModel.addBindingObject(bTimes, c);
        bindingModel.put(times, bTimes);
        
        // The binding of the '+' Node
        Binding<Node> bPlus = new BindingAdapter<Node>() {
            private static final long serialVersionUID = 1L;
            public void update(BindingContext<Node> context) {
                System.out.println("bPlus:update()");
                Node bound = getBoundObject();
                Set<Node> bindingObjects = getBindingObjects();
                bound.x = 0;
                for (Node node:bindingObjects) {
                    bound.x += node.x;
                }
            }
            public Object clone() {
                return super.clone();
            }
        };
        bindingModel.addBindingObject(bPlus, times);
        bindingModel.addBindingObject(bPlus, a);
        bindingModel.put(plus, bPlus);

        System.out.println(bTimes);
        System.out.println(bPlus);
        
        // Change and update
        Set<Node> changedObjects = new HashSet<Node>();
        changedObjects.add(b);
        BindingUtils.updateBindings(bindingModel, changedObjects, null);
        System.out.println("Result: " + plus.x);
        System.out.println(a + "+" + b + "*" + c + "=" + plus.x);
        
        // Another test
        changedObjects.clear();
        a.x = 4;
        changedObjects.add(a);
        BindingUtils.updateBindings(bindingModel, changedObjects, null);
        System.out.println("Result: " + plus.x);
        System.out.println(a + "+" + b + "*" + c + "=" + plus.x);
    }
}
