/*
 * Copyright (C) 2002-2010 Berthold Firmenich
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, look at "http://www.gnu.org/copyleft/gpl.html" or
 * write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 *
 * Contact address: berthold.firmenich@cademia.org
 * Homepage       : http://www.cademia.org/
 */

package cib.util.coll;

import java.util.Iterator;
import java.util.ListIterator;
import java.util.Map;
import java.util.Set;

import cib.util.NameSpace;

/**
 * Static Collection utilities.
 *
 * @author Berthold Firmenich, Torsten Richter
 * @version $Revision: 1.11 $, $Author: cvs-cad-firmenic $
 * @version $Date: 2010/01/13 15:05:11 $
 */
public class CollUtilities {

    // Sets
    public static <E> Set<E> subSet(Set<E> subSet, ObservableSet<E> superSet) {
        return new _SubSet<E>(subSet, superSet);
    }
    public static <E> Set<E> namedElementsSet(Set<E> set, NameSpace nameSpace) {
        return new _NamedElementsSet<E>(set, nameSpace);
    }
    public static <E> ObservableSet<E> observableSet(Set<E> set) {
        return new _ObservableSet<E>(set);
    }

    // Lists
    public static <E> SwappableList<E> orderedSet(ObservableSet<E> set) {
        return new _OrderedSet<E>(set);
    }
    
    // Maps
    public static <K, V> ReverseAccessMap<K, V> consistentHashMap(
            ReverseAccessMap<K, V> map, ObservableSet<K> keys,
            ObservableSet<V> values) {
        return new _ConsistentHashMap<K,V>(map, keys, values);
    }
    public static <K,V> ReverseAccessMap<K,V> reverseAccessMap(Map<K,V> map) {
        return new _ReverseAccessHashMap<K,V>(map);
    }
    public static <K, V> ReverseAccessMap<K, V> bijectiveReverseAccessMap(
            Map<K, V> map) {
        return new _InjectiveReverseAccessHashMap<K, V>(map);
    }

    // Filterable iterator
    public static <E> Iterator<E> filterableIterator(Iterator<E> iter, 
            IteratorFilter<E> filter){
        return new _FilterableIterator<E>(iter,filter);
    }
    public static <E> ListIterator<E> filterableListIterator(
            ListIterator<E> iter, IteratorFilter<E> filter){
        return new _FilterableListIterator<E>(iter,filter);
    }
    public static <E> ListIterator<E> multiFilterableListIterator(
            ListIterator<E> iter, IteratorFilter<E>[] filters){
        return new _MultiFilterableListIterator<E>(iter,filters);
    }
    public static <E> NamedListIterator<E> multiFilterableNamedListIterator(
            ListIterator<E> iter, IteratorFilter<E>[] filters){
        return new _MultiFilterableNamedListIteratorAdapter<E>(iter,filters);
    }
}