/* 
 * Copyright (C) 2002-2010 Berthold Firmenich
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, look at "http://www.gnu.org/copyleft/gpl.html" or
 * write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 * 
 * Contact address: berthold.firmenich@cademia.org
 * Homepage       : http://www.cademia.org/
 */

package cib.util.coll;

import java.util.ListIterator;
import java.util.NoSuchElementException;

/** 
 * A NamedListIterator extends the ListIterator interface by assigning unique
 * names to the list elements. For efficiency reasons unique names are
 * represented by integers.
 * 
 * @author Berthold Firmenich
 * @version $Revision: 1.5 $, $Author: cvs-cad-firmenic $
 * @version $Date: 2010/01/13 15:05:11 $
 */
public interface NamedListIterator<E> extends ListIterator<E> {
    /** 
     * Returns the name of the element that would be returned by a subsequent
     * call to <tt>previous()</tt>.
     *      
     * @return the name of the element that would be returned by a subsequent
     *  call to <tt>previous()</tt>.
     * @throws NoSuchElementException if the list iterator is at the beginning
     *  of the list
     */
    public int previousName() throws NoSuchElementException;

    /** 
     * Returns the name of the element that would be returned by a subsequent
     * call to <tt>next()</tt>.
     *      
     * @return the name of the element that would be returned by a subsequent
     *  call to <tt>next()</tt>.
     * @throws NoSuchElementException if the list iterator is at the end of the
     *  list
     */
    public int nextName() throws NoSuchElementException;
}