/*
 * Copyright (C) 2002-2010 Berthold Firmenich
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, look at "http://www.gnu.org/copyleft/gpl.html" or
 * write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 *
 * Contact address: berthold.firmenich@cademia.org
 * Homepage       : http://www.cademia.org/
 */

package cib.util.coll;

import java.io.Serializable;
import java.util.Collection;
import java.util.Iterator;
import java.util.Set;

/**
 * A wrapper for a set that is a subset of another set.
 *
 * @author Berthold Firmenich, Torsten Richter
 * @version $Revision: 1.10 $, $Author: cvs-cad-firmenic $
 * @version $Date: 2010/01/13 15:05:11 $
 */
class _SubSet<E> implements Set<E>, Serializable {

    private static final long serialVersionUID = 0L;

    protected ObservableSet<E> m_superSet;
    protected Set<E> m_subSet;

    // Constructors
    protected _SubSet(Set<E> subSet, ObservableSet<E> superSet) {
        if (superSet == null || subSet == null)
            throw new IllegalArgumentException();

        m_superSet = superSet;
        m_subSet = subSet;

        superSet.addListener(new ObservableSet.Listener<E>() {
            private static final long serialVersionUID = 0L;

            public void wasRemoved(Object o) {
                m_subSet.remove(o);
            }
        });
    }

    // Set implementation
    public boolean add(E e) {
        if (!m_superSet.contains(e))
            return false;
        return m_subSet.add(e);
    }

    public boolean addAll(Collection<? extends E> c) {
        if (!m_superSet.containsAll(c))
            return false;
        return m_subSet.addAll(c);
    }

    public void clear() {
        m_subSet.clear();
    }

    public boolean contains(Object o) {
        return m_subSet.contains(o);
    }

    public boolean containsAll(Collection<?> c) {
        return m_subSet.containsAll(c);
    }

    public boolean equals(Object o) {
        return m_subSet.equals(o);
    }

    public int hashCode() {
        return m_subSet.hashCode();
    }

    public boolean isEmpty() {
        return m_subSet.isEmpty();
    }

    public Iterator<E> iterator() {
        return m_subSet.iterator();
    }

    public boolean remove(Object o) {
        return m_subSet.remove(o);
    }

    public boolean removeAll(Collection<?> c) {
        return m_subSet.removeAll(c);
    }

    public boolean retainAll(Collection<?> c) {
        return m_subSet.retainAll(c);
    }

    public int size() {
        return m_subSet.size();
    }

    public Object[] toArray() {
        return m_subSet.toArray();
    }

    public <T> T[] toArray(T[] a) {
        return m_subSet.toArray(a);
    }

    public String toString() {
        return m_subSet.toString();
    }
}