/*
 * Copyright (C) 2002-2010 Berthold Firmenich
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, look at "http://www.gnu.org/copyleft/gpl.html" or
 * write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 *
 * Contact address: berthold.firmenich@cademia.org
 * Homepage       : http://www.cademia.org/
 */

package cib.util.featureLogic;

import java.io.Reader;
import java.io.StringReader;
import java.sql.SQLException;

import cib.util.featureLogic.interpreter.Interpreter;
import cib.util.featureLogic.interpreter.ParseException;
import cib.util.featureLogic.interpreter.TokenMgrError;

/**
 * The FeatureLogicAdapter is an abstract class. It implements the generic
 * methods of the FeatureLogic class.
 *
 * @author Berthold Firmenich
 * @version $Revision: 1.6 $, $Author: cvs-cad-firmenic $
 * @version $Date: 2010/01/13 15:05:11 $
 */
public abstract class FeatureLogicAdapter implements FeatureLogic {

    // ------------------------------------------------------------------------
    // Calculation of element sets from feature terms
    // ------------------------------------------------------------------------

    public ResultSetIterator interpret(Reader reader) throws SQLException,
            FeatureLogicException {

        Interpreter interpreter = new Interpreter(reader);
        interpreter.setFeatureLogic(this);
        interpreter.ReInit(reader);
        try {
            return resultSetIterator(interpreter.start());
        } catch (TokenMgrError e) {
            System.out.println(e);
        } catch (ParseException e) {
            System.out.println(e);
        } catch (SQLException e) {
            System.out.println(e);
        } catch (FeatureLogicException e) {
            System.out.println(e);
        }
        return null;
    }

    public ResultSetIterator interpret(String s) throws SQLException,
            FeatureLogicException {
        return interpret(new StringReader(s));
    }

    public String singleton(Reader reader) throws SQLException,
            FeatureLogicException {
        ResultSetIterator it = interpret(reader);
        String element = it.nextElement();
        it.close();
        return element;
    }

    public String singleton(String s) throws SQLException,
            FeatureLogicException {
        return singleton(new StringReader(s));
    }
}