/*
 * Copyright (C) 2002-2010 Berthold Firmenich
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, look at "http://www.gnu.org/copyleft/gpl.html" or
 * write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 *
 * Contact address: berthold.firmenich@cademia.org
 * Homepage       : http://www.cademia.org/
 */

package cib.util.featureLogic.db.mysql;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import cib.util.featureLogic.FeatureLogic;
import cib.util.featureLogic.FeatureLogicAdapter;
import cib.util.featureLogic.FeatureLogicException;
import cib.util.featureLogic.ResultSetIterator;
import cib.util.featureLogic.ResultSetIteratorImp;
import cib.util.featureLogic.SetDsc;
import cib.util.featureLogic.db.oracle.SetDscOracle;
import cib.util.featureLogic.interpreter.Interpreter;
import cib.util.featureLogic.interpreter.ParseException;
import cib.util.featureLogic.interpreter.TokenMgrError;

/**
 * This class implements the FeatureLogic for MySQL.
 *
 * @author Bertie Olivier
 * @version $Revision: 1.14 $, $Author: cvs-cad-firmenic $
 * @version $Date: 2010/01/13 15:05:11 $
 */
public class FeatureLogicMySql extends FeatureLogicAdapter {

    private Connection m_con = null;


   /**
    * @param con the JDBC database connection
    */
    public FeatureLogicMySql(Connection con) {
        m_con = con;
    }

//  ------------------------------------------------------------------------
//  Data manipulation
//  ------------------------------------------------------------------------

    public boolean isBatchMode() {
        return false;
    }

    public void setBatchMode(boolean batchMode) {
    }

    public void executeBatch() throws SQLException {
    }

    public void clear(boolean onlyTransient) throws SQLException {
        if (m_con == null)
            return;

        Statement stmt = null;
        stmt = m_con.createStatement();
        stmt.addBatch("DELETE FROM ATOM");
        stmt.addBatch("DELETE FROM DOMAIN");
        stmt.addBatch("DELETE FROM FEATURE");
        stmt.addBatch("DELETE FROM RELSLOT");
        stmt.addBatch("DELETE FROM NextAvailableId");
        stmt.addBatch("INSERT INTO NextAvailableId (ID) VALUES (1)");
        stmt.executeBatch();
        stmt.close();
    }

    private long _generateUniqueId() throws SQLException {

        // Get current component ID
        String sql = "SELECT id FROM NextAvailableId";
        Statement stmt = m_con.createStatement();
        ResultSet rs = stmt.executeQuery(sql);
        rs.next();
        long idCmp = rs.getLong(1);
        rs.close();
        stmt.close();

        // Update component ID
        long idCmpNext = idCmp + 1;
        sql =
            "UPDATE NextAvailableId SET id=" + idCmpNext + " WHERE id=" +
            idCmp;
        executeUpdate(sql);

        return idCmp;
    }

    private boolean _storeElement(String element) throws SQLException {
        if (! isElement(element)) {
            String sql = "INSERT INTO Domain VALUES ('" + element + "')";
            executeUpdate(sql);
            return true;
        }
        return false;
    }

    public String addElement() throws SQLException {
        long id = _generateUniqueId();
        String element = "_" + id;
        _storeElement(element);
        return element;
    }

    public boolean addElement(String name) throws SQLException {
        return _storeElement(name);
    }

    public void removeElement(String id) throws SQLException {
        String sql = "DELETE * FROM Domain WHERE element='" + id + "'";
        executeUpdate(sql);
    }

    public void defineFeature(String feature) throws SQLException {
        if (! isFeature(feature)) {
            String sql = "INSERT INTO Feature VALUES ('" + feature + "')";
            executeUpdate(sql);
        }
    }

    public void addFeature(String element, String feature, String value)
        throws SQLException, FeatureLogicException {

        // Ensure consistency of the data model
        checkElement(element);
        checkUsableAsNonPrimitiveElement(element);
        checkElement(value);
        checkFeature(feature);

        // Add feature pair
        String sql =
            "INSERT INTO RelSlot (object, feature, value) " +
            "VALUES ('" + element + "','" + feature + "','" + value + "')";
        executeUpdate(sql);
    }

    public void changeFeature(String element, String feature, String value)
        throws SQLException, FeatureLogicException {

        // Ensure consistency of the data model
        checkElement(value);

        // Modify feature pair
        String sql =
            "UPDATE RelSlot SET (value)='" + value + "' " +
            "WHERE object='" + element + "' AND feature='" + feature + "'";
        executeUpdate(sql);
    }

    public void removeFeature(String element, String feature)
        throws SQLException, FeatureLogicException {
        throw new UnsupportedOperationException();
    }

    public String findOrCreateAtom(int value)
        throws SQLException, FeatureLogicException {
        return _findOrCreateLiteral(String.valueOf(value));
    }

    public String findOrCreateAtom(long value)
        throws SQLException, FeatureLogicException {
        return _findOrCreateLiteral(String.valueOf(value) + "L");
    }

    public String findOrCreateAtom(double value)
        throws SQLException, FeatureLogicException {
        return _findOrCreateLiteral(String.valueOf(value));
    }

    public String findOrCreateAtom(String value)
        throws SQLException, FeatureLogicException {
        return _findOrCreateLiteral("\"" + value + "\"");
    }

    public String findOrCreateAtom(Date date)
        throws SQLException, FeatureLogicException {
        return _findOrCreateLiteral(String.valueOf(date.getTime()) + "DATE");
    }

    private String _findOrCreateLiteral(String literal) throws SQLException {

        // Does the literal already exist?
        String sql =
            "SELECT element FROM Atom WHERE value = '" + literal + "'";
        Statement stmt = m_con.createStatement();
        ResultSet rs = stmt.executeQuery(sql);
        if (rs.next()) {
            String element = rs.getString(1);
            rs.close();
            stmt.close();
            return element;
        }
        stmt.close();

        // Create a new literal
        String element = addElement();
        sql =
            "INSERT INTO Atom (element, value) " +
            "VALUES ('" + element + "','" + literal + "')";
        executeUpdate(sql);

        // Return the element
        return element;
    }

//  ------------------------------------------------------------------------
//  Miscellaneous
//  ------------------------------------------------------------------------

    public boolean isElement(String element) throws SQLException {
        String sql =
            "SELECT Count(*) AS n FROM Domain " +
            "WHERE Domain.element='" + element + "'";
        Statement stmt = m_con.createStatement();
        ResultSet rs = stmt.executeQuery(sql);

        rs.next();
        int n = rs.getInt(1);

        rs.close();
        stmt.close();

        return n != 0;
    }

    public boolean isUsedAsPrimitiveElement(String element)
    throws SQLException {
        String sql =
            "SELECT Count(*) AS n FROM Atom " +
            "WHERE Atom.element='" + element + "'";
        Statement stmt = m_con.createStatement();
        ResultSet rs = stmt.executeQuery(sql);

        rs.next();
        int n = rs.getInt(1);

        rs.close();
        stmt.close();

        return n != 0;
    }

    public boolean isUsedAsNonPrimitiveElement(String element)
    throws SQLException {
        String sql =
            "SELECT Count(*) AS n FROM RelSlot " +
            "WHERE RelSlot.object='" + element + "'";
        Statement stmt = m_con.createStatement();
        ResultSet rs = stmt.executeQuery(sql);

        rs.next();
        int n = rs.getInt(1);

        rs.close();
        stmt.close();

        return n != 0;
    }

    public boolean isFeature(String feature) throws SQLException {
        String sql =
            "SELECT Count(*) AS n FROM Feature " +
            "WHERE Feature.feature='" + feature + "'";
        Statement stmt = m_con.createStatement();
        ResultSet rs = stmt.executeQuery(sql);

        rs.next();
        int n = rs.getInt(1);

        rs.close();
        stmt.close();

        return n != 0;
    }

    public String getFeatureValue(String element, String feature)
    throws SQLException, FeatureLogicException {
        checkElement(element);
        checkFeature(feature);

        String sql =
            "SELECT value " +
            "FROM RelSlot " +
            "WHERE object='" + element + "' AND feature='" + feature + "'";

        Statement stmt = m_con.createStatement();
        ResultSet rs = stmt.executeQuery(sql);
        String value = rs.next() ? rs.getString(1) : null;
        rs.close();
        stmt.close();

        return value;
    }

    public void getFeatureValues(String element, Map<String,String> features)
    throws SQLException, FeatureLogicException {
        checkElement(element);

        String sql =
            "SELECT feature, value " +
            "FROM RelSlot " +
            "WHERE object='" + element + "'";

        Statement stmt = m_con.createStatement();
        ResultSet rs = stmt.executeQuery(sql);

        features.clear();
        while (rs.next()) {
            features.put(rs.getString(1), rs.getString(2));
        }

        rs.close();
        stmt.close();
    }

    public Map<String,Map<String,String>> getFeatureValues(Set<String> elements)
    throws SQLException, FeatureLogicException {
        Map<String,Map<String,String>> map = new HashMap<String,Map<String,String>>();

        for (String element : elements) {
            Map<String,String> features = new HashMap<String,String>();
            getFeatureValues(element, features);

            map.put(element, features);
        }
        return map;
    }

    public Object getAtom(String prmElement)
    throws SQLException, FeatureLogicException {
        checkElement(prmElement);

        // Get value from database
        String sql =
            "SELECT value " +
            "FROM Atom " +
            "WHERE element='" + prmElement + "'";

        Statement stmt = m_con.createStatement();
        ResultSet rs = stmt.executeQuery(sql);

        if (! rs.next())
            throw new
                FeatureLogicException("getAtom(): no primitive element");

        String value = rs.getString(1);
        rs.close();
        stmt.close();

        // Convert string value to typed value
        Object typedValue;
        if (value.startsWith("\""))
            typedValue = value.substring(1, value.length() - 1);
        else if (-1 != value.indexOf('.'))
            typedValue = new Double(value);
        else if (value.endsWith("L")) {
            value = value.substring(0, value.length() - 1);
            typedValue = new Long(value);
        }
        else if (value.endsWith("DATE")) {
            value = value.substring(0, value.length() - 4);
            typedValue = new Date(new Long(value));
        }
        else
            typedValue = new Integer(value);

        return typedValue;
    }

    public Map<String,Object> getAllAtoms() throws SQLException,
            FeatureLogicException {
        throw new UnsupportedOperationException();
    }

//  ------------------------------------------------------------------------
//  Feature Logic algebra
//  ------------------------------------------------------------------------

    public SetDsc top() {
        return new SetDscMySql(
            "SELECT element FROM Domain"
        );
    }

    public SetDsc bottom() {
        return new SetDscMySql(
            "SELECT element FROM Domain WHERE 0 = 1"
        );
    }

    public SetDsc element(String x) {
        return new SetDscMySql(
            "SELECT element FROM Domain WHERE element = '" + x + "'"
        );
    }

    public SetDsc primitiveElement(int atom) {
        return new SetDscMySql(
            "SELECT element FROM Atom WHERE value = '" + atom + "'"
        );
    }

    public SetDsc primitiveElement(long atom) {
        return new SetDscOracle(
            "SELECT element FROM Atom WHERE value = '" + atom + "'"
        );
    }

    public SetDsc primitiveElement(double atom) {
        return new SetDscMySql(
            "SELECT element FROM Atom WHERE value = '" + atom + "'"
        );
    }

    public SetDsc primitiveElement(String atom) {
        return new SetDscMySql(
            "SELECT element FROM Atom WHERE value = '\"" + atom + "\"'"
        );
    }

    public SetDsc extract(String f, SetDsc s) {
        return new SetDscMySql(
            "SELECT DISTINCT RelSlot.value AS element" +
            " FROM (" + s + ") S, RelSlot" +
            " WHERE RelSlot.feature = '" + f + "' AND" +
            " Relslot.object = S.element"
        );
    }

    public SetDsc selection(String f, SetDsc s) {
        return new SetDscMySql(
            "SELECT DISTINCT Relslot.object AS element" +
            " FROM (" + s + ") S, RelSlot" +
            " WHERE Relslot.feature = '" + f + "' AND" +
            " Relslot.value = S.element"
        );
    }

    public SetDsc existence(String f) {
        return new SetDscMySql(
            "SELECT DISTINCT Relslot.object AS element" +
            " FROM RelSlot WHERE Relslot.feature = '" + f + "'"
        );
    }

    public SetDsc divergence(String f) {
        return new SetDscMySql(
            "SELECT element FROM Domain" +
            " MINUS" +
            " SELECT DISTINCT object AS element FROM RelSlot" +
            " WHERE feature = '" + f + "'"
        );
    }

    public SetDsc agreement(String f, String g) {
        return new SetDscMySql(
            "SELECT DISTINCT A.object AS element" +
            " FROM RelSlot A, RelSlot B" +
            " WHERE" +
            " A.object = B.object AND" +
            " A.feature = '" + f + "' AND" +
            " B.feature = '" + g + "' AND" +
            " A.value = B.value"
        );
    }

    public SetDsc disagreement(String f, String g) {
        return new SetDscMySql(
            "SELECT DISTINCT A.object AS element" +
            " FROM RelSlot A, RelSlot B" +
            " WHERE" +
            " A.object = B.object AND" +
            " A.feature = '" + f + "' AND" +
            " B.feature = '" + g + "' AND" +
            " A.value <> B.value"
        );
    }

    public SetDsc union(SetDsc s, SetDsc t) {
        return new SetDscMySql(
            "(" + s + ") UNION (" + t + ")"
        );
    }

    public SetDsc intersection(SetDsc s, SetDsc t) {
        return new SetDscMySql(
            "(" + s + ") INTERSECT (" + t + ")"
        );
    }

    public SetDsc difference(SetDsc s, SetDsc t) {
        return new SetDscMySql(
            "(" + s + ") MINUS (" + t + ")"
        );
    }

    public SetDsc complement(SetDsc s) {
        return new SetDscMySql(
            "SELECT element FROM Domain MINUS (" + s + ")"
        );
    }

    public SetDsc implication(SetDsc s, SetDsc t) {
       return union(complement(s), t);
    }

    public SetDsc equivalence(SetDsc s, SetDsc t) {
        return intersection(implication(s, t), implication(t, s));
    }

    public int cardinality(SetDsc s)
    throws SQLException, FeatureLogicException {

        String sql = "SELECT Count(*) AS n FROM (" + s + ")";

        Statement stmt = m_con.createStatement();
        ResultSet rs = stmt.executeQuery(sql);

        int n = 0;
        if (rs.next())
            n = rs.getInt(1);

        rs.close();
        stmt.close();

        return n;
    }

//  ------------------------------------------------------------------------
//  Iteration of the result set
//  ------------------------------------------------------------------------

    public ResultSetIterator resultSetIterator(SetDsc s) throws SQLException {
        Statement statement = m_con.createStatement();
        ResultSet resultSet = statement.executeQuery(
//            "SELECT element FROM Domain WHERE 0 = 1 UNION (" + s.toString() + ")"
            s.toString()
        );
        return new ResultSetIteratorImp(statement, resultSet);
    }

//  ------------------------------------------------------------------------
//  Helpers
//  ------------------------------------------------------------------------

   /**
    * Excecutes a SQL Update statement
    * @param sql SQL string
    */
    private void executeUpdate(String sql) throws SQLException {
        Statement stmt = m_con.createStatement();
        stmt.executeUpdate(sql);
        stmt.close();
    }

   /**
     * Checks whether an element exists in the Domain. On error an exception is
     * thrown.
     *
     * @param element the element to be checked
     */
    private void checkElement(String element) throws SQLException,
            FeatureLogicException {

        if (!isElement(element))
            throw new FeatureLogicException("Invalid element " + element);
    }

    /**
     * Checks whether an element is usable as non-primitive element. If the
     * element is already used as primitive an exception is thrown.
     *
     * @param element the element to be checked
     */
    private void checkUsableAsNonPrimitiveElement(String element)
            throws SQLException, FeatureLogicException {

        if (isUsedAsPrimitiveElement(element))
            throw new FeatureLogicException(
                "Attempt to use the primitive element " + element
                        + " as non-primitive element");
    }

    /**
     * Checks whether a feature is defined. On error an exception is thrown.
     *
     * @param feature the feature name to be checked
     */
    private void checkFeature(String feature) throws SQLException,
            FeatureLogicException {

        if (!isFeature(feature))
            throw new FeatureLogicException("Invalid feature " + feature);
    }

    public static void main(String[] args) throws ClassNotFoundException,
            SQLException {
        // Commandline args
        if (args.length == 0) {
            System.out
                .println("usage:  FeatureLogicMySql -dbURL <JDBC database URL>");
            System.out
                .println("sample: FeatureLogicMySql -dbURL jdbc:mysql://127.0.0.1/FeatureLogic");
            System.exit(0);
        }

        // Get command line switches
        String databaseURL = null;
        for (int i = 0; i < args.length; i++) {
            if (args[i].equals("-dbURL"))
                databaseURL = args[++i];
        }
        if (databaseURL == null) {
            System.out.println("Please enter the database URL");
            System.exit(1);
        }

        // Database constants
        final String driver = "com.mysql.jdbc.Driver";
        final String user = "bertie";
        final String password = "*****";

        // Load JDBC driver
        System.out.println("Loading driver " + driver + " ...");
        Class.forName(driver);

        // Connect to database
        System.out.println("Connecting to database " + databaseURL + " ...");
        Connection con =
            DriverManager.getConnection(databaseURL, user, password);

        Interpreter interpreter = new Interpreter(System.in);
        FeatureLogic featureLogic = new FeatureLogicMySql(con);
        interpreter.setFeatureLogic(featureLogic);
        while (true) {
            interpreter.ReInit(System.in);
            System.out.print("Enter Feature Term: ");
            System.out.flush();
            try {
                SetDsc setDsc = interpreter.start();
                if (setDsc == null)
                    System.exit(0);
                System.out.println(setDsc); // Temporary show string
                ResultSetIterator rsi = featureLogic.resultSetIterator(setDsc);
                while (rsi.hasNext()) {
                    System.out.println("\t" + rsi.next());
                }
            } catch (TokenMgrError e) {
                System.out.println(e);
            } catch (ParseException e) {
                System.out.println(e);
            } catch (SQLException e) {
                System.out.println(e);
            } catch (FeatureLogicException e) {
                System.out.println(e);
            }
        }
    }
}