/*
 * Copyright (C) 2002-2010 Berthold Firmenich
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, look at "http://www.gnu.org/copyleft/gpl.html" or
 * write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 *
 * Contact address: berthold.firmenich@cademia.org
 * Homepage       : http://www.cademia.org/
 */

package cib.util.featureLogic.db.postgresql;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

/**
 * @author Torsten Richter
 * @version $Revision: 1.3 $, $Author: cvs-cad-firmenic $
 * @version $Date: 2010/01/13 15:05:11 $
 */
public class SetupPostgreSQLDatabase {
    private static final String DATABASE_URL = "jdbc:postgresql://141.54.182.8:5432/featurelogic";
    private static final String DATABASE_NAME = "featurelogic";
    private static final String DRIVER = "org.postgresql.Driver";
    private static final String USER = "p_bertie";
    private static final String PASSWORD = "bertie";

    private static Connection s_con;
    private static Statement s_stmt;

    private static void openConnection(){
        try {
            // Load JDBC DRIVER
            Class.forName(DRIVER);
        } catch (ClassNotFoundException e) {
            System.err.println(DRIVER +" not found");
            e.printStackTrace();
        }
        try {
            // Connect to database
            s_con = DriverManager.getConnection(DATABASE_URL, USER, PASSWORD);
            s_stmt = s_con.createStatement();
        } catch (SQLException e1) {
           System.err.println("Could not connect to the url : "+DATABASE_URL);
           e1.printStackTrace();
        }
    }

    public static int update(String sql) {
        openConnection();
        int retValue = -1;
        try {
            retValue = s_stmt.executeUpdate(sql);
        } catch (SQLException e) {
            //e.printStackTrace();
            System.err.println(e.getMessage());
            closeConnection();
        }
        closeConnection();
        return retValue;
    }

    public static ResultSet runQuery(String sql){
        openConnection();
        ResultSet rSet;
        try {
            rSet = s_stmt.executeQuery(sql);
            return rSet;
        } catch (SQLException e) {
            e.printStackTrace();
        }
        closeConnection();
        return null;
    }

    private static String format(String s){
        s = " "+s;
        while(s.length()<20)s +=' ';
        s +="|";
        return s;
    }

    private static String line(int cols){
        String s = "\t+";
        for(int i = 0; i < cols; i++ ){
            for(int j = 0; j < 20; j++ ){
                s += "-";
            }
            s +="+";
        }
        return s;
    }

    private static void printTable(String tbl_name){
        ResultSet rset = runQuery("SELECT * FROM "+tbl_name+";");
        try {
            System.out.println(tbl_name);
            System.out.println(line(rset.getMetaData().getColumnCount()));
            System.out.print("\t|");
            for (int i = 1; i <= rset.getMetaData().getColumnCount(); i++)
                System.out.print(format(rset.getMetaData().getColumnName(i).toString()));
            System.out.print("\n");
            System.out.println(line(rset.getMetaData().getColumnCount()));
            while(rset.next()){
                System.out.print("\t|");
                for (int i = 1; i <= rset.getMetaData().getColumnCount(); i++)
                    System.out.print(format(rset.getObject(i).toString()));
                System.out.print("\n");
            }
            System.out.println(line(rset.getMetaData().getColumnCount()));
        } catch (SQLException e) {
            e.printStackTrace();
        }
    }

    private static void closeConnection(){
        try {
            s_stmt.close();
            s_con.close();
        } catch (SQLException sqle) {
            System.err.println("Could not disconnect from "+ DATABASE_URL);
        }
    }

    /*private static void createDatabase(){
        try {
            Class.forName(DRIVER);// Load JDBC DRIVER
        } catch (ClassNotFoundException e) {
            System.err.println(DRIVER + " not found");
        }
        try {
            // Connect to database
            s_con = DriverManager.getConnection(DATABASE_URL,USER, PASSWORD);
            s_stmt = s_con.createStatement();
            s_stmt.executeUpdate("DROP DATABASE " + DATABASE_NAME);
            System.out.println("Drop database... " + DATABASE_NAME);
        } catch (SQLException e1) {}
        try {
            System.out.println("Create database... " + DATABASE_NAME);
            s_stmt.executeUpdate("CREATE DATABASE " + DATABASE_NAME);
        } catch (SQLException sqle) {}
        closeConnection();
    }*/

    private static void createTables(){
        String sql;

        // Create tables
        System.out.println("Creating TABLES...");

        // Atom
        System.out.println("\t... Atom");
        sql = "CREATE TABLE Atom(" +
                "element varchar(10)  NOT NULL," +
                "value   varchar(150) NOT NULL," +
                "PRIMARY KEY (element)" +
              ") WITHOUT OIDS TABLESPACE intercad";
        update(sql);

        // Domain
        System.out.println("\t... Domain");
        sql = "CREATE TABLE Domain(" +
                "element varchar(100) NOT NULL," +
                "PRIMARY KEY (Element)" +
              ") WITHOUT OIDS TABLESPACE intercad";
        update(sql);

        // Feature
        System.out.println("\t... Feature");
        sql = "CREATE TABLE Feature(" +
                "feature varchar(20) NOT NULL," +
                "PRIMARY KEY (feature)" +
              ") WITHOUT OIDS TABLESPACE intercad";
        update(sql);

        // RelSlot
        System.out.println("\t... RelSlot");
        sql = "CREATE TABLE RelSlot ("+
                "object  varchar(20)  NOT NULL," +
                "feature varchar(20)  NOT NULL," +
                "value   varchar(100) NOT NULL" +
              ") WITHOUT OIDS TABLESPACE intercad";
        update(sql);

        System.out.println("...done!");
    }

    private static void insertTestData(){
        String sql;
        System.out.println("Populating tables");
        // Atom
        System.out.println("\t... Atom");
        sql = "INSERT INTO Atom VALUES ('one','1');" +
              "INSERT INTO Atom VALUES ('two','2');" +
              "INSERT INTO Atom VALUES ('four','4');" +
              "INSERT INTO Atom VALUES ('six','6');" +
              "INSERT INTO Atom VALUES ('zero_three','0.3');" +
              "INSERT INTO Atom VALUES ('zero_seven','0.7');" +
              "INSERT INTO Atom VALUES ('one_nine','1.9');" +
              "INSERT INTO Atom VALUES ('peugeot','Peugeot');" +
              "INSERT INTO Atom VALUES ('skoda','SkodaFabia');" +
              "INSERT INTO Atom VALUES ('iveco','Iveco');";

        update(sql);

        // Domain
        System.out.println("\t... Domain");
        sql = "INSERT INTO Domain VALUES ('bicycle');" +
              "INSERT INTO Domain VALUES ('car');" +
              "INSERT INTO Domain VALUES ('truck');" +
              "INSERT INTO Domain VALUES ('one');" +
              "INSERT INTO Domain VALUES ('two');" +
              "INSERT INTO Domain VALUES ('four');" +
              "INSERT INTO Domain VALUES ('six');";
              //"INSERT INTO Domain VALUES ('zero_three');" +
              //"INSERT INTO Domain VALUES ('zero_seven');" +
              //"INSERT INTO Domain VALUES ('one_nine');" +
              //"INSERT INTO Domain VALUES ('iveco');" +
              //"INSERT INTO Domain VALUES ('peugeot');" +
              //"INSERT INTO Domain VALUES ('skoda');";
        update(sql);

        // Feature
        System.out.println("\t... Feature");
        sql = //"INSERT INTO Feature VALUES ('cw');" +
              "INSERT INTO Feature VALUES ('passengers');" +
              "INSERT INTO Feature VALUES ('type');" +
              "INSERT INTO Feature VALUES ('wheels');";
        update(sql);

        // RelSlot
        System.out.println("\t... RelSlot");
        sql = //"INSERT INTO Relslot VALUES ('bicycle','cw','zero_seven');" +
              "INSERT INTO Relslot VALUES ('bicycle','passengers','one');" +
              //"INSERT INTO Relslot VALUES ('bicycle','type','peugeot');" +
              "INSERT INTO Relslot VALUES ('bicycle','wheels','two');" +
              //"INSERT INTO Relslot VALUES ('car','cw','zero_three');" +
              "INSERT INTO Relslot VALUES ('car','passengers','four');" +
              //"INSERT INTO Relslot VALUES ('car','type','skode');" +
              "INSERT INTO Relslot VALUES ('car','wheels','four');" +
              //"INSERT INTO Relslot VALUES ('truck','cw','one_nine');" +
              "INSERT INTO Relslot VALUES ('truck','passengers','two');" +
              //"INSERT INTO Relslot VALUES ('truck','type','iveco');" +
              "INSERT INTO Relslot VALUES ('truck','wheels','six');" ;
        update(sql);

        System.out.println("...done!");
    }

    private static void printTestData(){
        System.out.println("Show tables...");
        printTable("Atom");
        printTable("Domain");
        printTable("Feature");
        printTable("RelSlot");
        System.out.println("...done!");
    }

    public static void main(String[] args) {
        //createDataBase();
        //createTables();
        insertTestData();
        //printTestData();
    }
}
