/* 
 * Copyright (C) 2002-2010 Berthold Firmenich
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, look at "http://www.gnu.org/copyleft/gpl.html" or
 * write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 * 
 * Contact address: berthold.firmenich@cademia.org
 * Homepage       : http://www.cademia.org/
 */

package cib.util.update;


/** 
 * This interface provides definitions for objects that represent a directed graph.
 *
 * @author Christian Koch
 * @version $Revision: 1.7 $, $Author: cvs-cad-firmenic $
 * @version $Date: 2010/01/13 15:05:11 $
 * @see cib.util.update._Graph
 * @see cib.util.update.Node
 * @see cib.util.update.GraphException
 */
@Deprecated
public interface Graph {

  /** 
   * Access method for the graph's nodes.
   * @return an array of the graph's nodes
   */
  public Node[] getNodes();

  /** 
   * Access method for the successor nodes of a specified node.
   * @param node the specified node
   * @return an array of the successor nodes
   */
  public Node[] getEdges(Node node);

  /** 
   * Adds a specified node to the graph.
   * @param node the specified node
   * @return <code>true</code> if the node was added
   */
  public boolean addNode(Node node);

  /** 
   * Removes a specified node from the graph.
   * @param node the specified node
   * @return <code>true</code> if the node was removed
   */
  public boolean removeNode(Node node);

  /** 
   * Adds an edge to the graph.
   * @param src the specified source node
   * @param dest the specified destination node
   * @return <code>true</code> if the edge was added
   * @throws GraphException a graph exception
   */
  public boolean addEdge(Node src, Node dest) throws GraphException;

  /** 
   * Removes an edge from the graph.
   * @param src the specified source node
   * @param dest the specified destination node
   * @return <code>true</code> if the edge was removed
   */
  public boolean removeEdge(Node src, Node dest);

  /** 
   * Determines if the graph contains a specified node.
   * @param node the specified node
   * @return <code>true</code> if the graph contains the specified node
   */
  public boolean containsNode(Node node);

  /** 
   * Determines if the graph is cyclic.
   * @return <code>true</code> if the graph is cyclic
   */
  public boolean hasCycle();

}//end of class