/* 
 * Copyright (C) 2002-2010 Berthold Firmenich
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, look at "http://www.gnu.org/copyleft/gpl.html" or
 * write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 * 
 * Contact address: berthold.firmenich@cademia.org
 * Homepage       : http://www.cademia.org/
 */

package cib.util.update;

import java.io.Serializable;
import java.util.ArrayList;

/** 
 * This class represents a directed graph.
 *
 * @author Christian Koch
 * @version $Revision: 1.9 $, $Author: cvs-cad-firmenic $
 * @version $Date: 2010/01/13 15:05:11 $
 * @see cib.util.update.Graph
 * @see cib.util.update.Node
 */
@Deprecated
public class _Graph implements Graph, Serializable {
    
    private static final long serialVersionUID = 0L;

    /** 
     * The list of the graph's nodes.
     */
    private ArrayList<Node> m_nodes = null;
    
    /** 
     * The cycle property.
     */
    private boolean m_hasCycle = false;
    
    /** 
     * Standard constructor.
     */
    public _Graph() {
        this.m_nodes = new ArrayList<Node>();
    }
    
    /** 
     * Access method for the graph's nodes.
     * @return an array of the graph's nodes
     */
    public Node[] getNodes() {
        Node[] _nodes = new Node[this.m_nodes.size()];
        for (int i=0; i < _nodes.length; i++) {
            _nodes[i] = this.m_nodes.get(i);
        }
        return _nodes;
    }
    
    /** 
     * Access method for the successor nodes of a specified node.
     * @param node the specified node
     * @return an array of the successor nodes
     */
    public Node[] getEdges(Node node) {
        if (! this.m_nodes.contains(node))
            throw new IllegalArgumentException("Node does not exists.");
        ArrayList<Node> _srcEdges = node.getEdges();
        Node[] _edges = new Node[_srcEdges.size()];
        for (int i=0; i < _edges.length; i++) {
            _edges[i] = _srcEdges.get(i);
            //System.out.println(((TestObject)_edges[i]).getID());
        }
        return _edges;
    }
    
    /** 
     * Adds a specified node to the graph.
     * @param node the specified node
     * @return <code>true</code> if the node was added
     */
    public boolean addNode(Node node) {
        return this.m_nodes.add(node);
        //System.out.println( "Added: "+((TestObject)vertex.getKey()).getID() );
    }
    
    /** 
     * Removes a specified node from the graph.
     * @param node the specified node
     * @return <code>true</code> if the node was removed
     */
    public boolean removeNode(Node node) {
        if (this.m_nodes.contains(node)) {
            ArrayList<Node> _edges;
            for (int i=0; i < this.m_nodes.size(); i++) {
                _edges = this.m_nodes.get(i).getEdges();
                for (int j=0; j < _edges.size(); j++) {
                    _edges.remove(node);
                }
            }
            return this.m_nodes.remove(node);
        }
        return false;
    }
    
    /** 
     * Adds an edge to the graph.
     * @param src the specified source node
     * @param dest the specified destination node
     * @return <code>true</code> if the edge was added
     * @throws GraphException a graph exception
     */
    public boolean addEdge(Node src, Node dest) throws GraphException {
        if (!this.m_nodes.contains(src))
            throw new IllegalArgumentException("Source node does not exist.");
        if (!this.m_nodes.contains(dest))
            throw new IllegalArgumentException(
                "Destination node does not exist.");
        ArrayList<Node> _edges = src.getEdges();
        if ( _edges.contains(dest))
            return false;
        
        _edges.add(dest);
        //test for cycles
        if (this.hasCycle()) {
            _edges.remove(dest);
            //String s = ("("+src.toString()+","+dest.toString()+")");
            throw new GraphException(
                "Edge (" + src + "," + dest +
                ") not added, because graph would be cyclic!");
        }
        return true;
    }
    
    /** 
     * Removes an edge from the graph.
     * @param src the specified source node
     * @param dest the specified destination node
     * @return <code>true</code> if the edge was removed
     */
    public boolean removeEdge(Node src, Node dest) {
        if ((this.m_nodes.contains(src)) && (this.m_nodes.contains(dest))) {
            return src.getEdges().remove(dest);
        }
        throw new IllegalArgumentException();
    }
    
    /** 
     * Determines if the graph contains a specified node.
     * @param node the specified node
     * @return <code>true</code> if the graph contains the specified node
     */
    public boolean containsNode(Node node) {
        return this.m_nodes.contains(node);
    }
    
    /** 
     * Determines if there is a cycle on the basis of a specified node.
     * @param node the specified node
     */
    private void hasCycle(Node n) {
        //set 1-state for this node
        n.setState(1);
        for (int i=0; i < n.getEdges().size(); i++) {
            if (n.getEdges().get(i).getState() == 1) {
                this.m_hasCycle = true;
                return;
            }
            else if (n.getEdges().get(i).getState() == 0)
                this.hasCycle(n.getEdges().get(i));
            //System.out.println(this.m_hasCycle);
        }
        n.setState(2);
    }
    
    /** 
     * Determines if the graph is cyclic.
     * @return <code>true</code> if the graph is cyclic
     */
    public boolean hasCycle() {
        this.m_hasCycle = false;
        //set 0-state for all vertices
        for (int i=0; i < this.m_nodes.size(); i++) {
            this.m_nodes.get(i).setState(0);
        }
        for (int i=0; i < this.m_nodes.size(); i++) {
            if (this.m_nodes.get(i).getState() == 0)
                //System.out.println("Test Knoten "+((TestObject)((Vertex)this.m_vertices.get(i)).getKey()).getID());
                this.hasCycle( this.m_nodes.get(i) );
            if (this.m_hasCycle) break;
            //System.out.println(i);
        }
        return this.m_hasCycle;
        //return this.hasCycle((Vertex)this.m_vertices.get(1));
    }
    
    /** 
     * Access method for the m_nodes property.
     * @return the current value of the m_nodes property
     */
    public ArrayList<Node> getNodeList() {
        return this.m_nodes;
    }
    
}//end of class