/* 
 * Copyright (C) 2009-2010 Berthold Firmenich
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, look at "http://www.gnu.org/copyleft/gpl.html" or
 * write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 * 
 * Contact address: berthold.firmenich@cademia.org
 * Homepage       : http://www.cademia.org/
 */

package cib.util.view;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.SwingUtilities;
import javax.swing.plaf.basic.BasicComboPopup;

/** 
 * At this writing objects of class JPopupMenu were not scrollable. This class
 * represents a very basic scrollable popup menu. The class should be deprecated
 * as soon as JPopupMenus can be scrolled.
 * 
 * @author Berthold Firmenich
 * @version $Revision: 1.7 $, $Author: cvs-cad-firmenic $
 * @version $Date: 2010/02/23 14:00:46 $
 */
public class ScrollableJPopupMenu implements ItemListener {
    private static final long serialVersionUID = 1L;

    private JComboBox m_comboBox;
    private JPopupMenu m_popupMenu;
    
    public static void main(String[] args) {
        final JFrame frame = new JFrame("Test");
        frame.setSize(300, 200);
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        final JButton button = new JButton("Press");
        frame.add(button, BorderLayout.NORTH);

        final ScrollableJPopupMenu m = new ScrollableJPopupMenu();
            button.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                m.removeAll();
                for (int i = 0; i < 100; i++) {
                    JMenuItem item = new JMenuItem("item " + i);
                    item.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                            m.setVisible(false);
                        }
                    });
                    m.add(item);
                }
                m.show(button, 0, 0);
            } 
        });

        frame.setVisible(true);
    }
    
    public ScrollableJPopupMenu() {
        _init();
    }

    private void _init() {
        m_comboBox = new JComboBox();
        m_popupMenu = new BasicComboPopup(m_comboBox);
    }
    
    public void itemStateChanged(ItemEvent e) {
        if (e.getStateChange() == ItemEvent.SELECTED) {
			ComboBoxItem item = (ComboBoxItem)e.getItem();
            item.getItem().doClick();
        }
    }
    
    public JMenuItem add(JMenuItem menuItem) {
        m_comboBox.addItem(new ComboBoxItem(menuItem));
        return menuItem;
    }

    public void addSeparator() {
        JMenuItem menuItem = new JMenuItem(
        	"\u2500\u2500\u2500\u2500\u2500\u2500");
        m_comboBox.addItem(new ComboBoxItem(menuItem));
    }
    
    public void removeAll() {
        _init();
    }
    
    public int getSelectedIndex() {
        return m_comboBox.getSelectedIndex();
    }
    
    public boolean isVisible() {
        return m_popupMenu.isVisible();
    }
    public void show(final Component invoker, final int x, final int y) {
        m_comboBox.setSelectedIndex(-1);
        m_comboBox.addItemListener(this);
        m_popupMenu.setPreferredSize(_preferredPopupMenuSize());
        SwingUtilities.invokeLater(new Thread() {
        	public void run() {
        		m_popupMenu.show(invoker, x, y);
        	}
        });
    }

    public void setVisible(boolean b) {
        m_comboBox.setSelectedIndex(-1);
        if (b)
            m_comboBox.addItemListener(this);
        else
            m_comboBox.removeItemListener(this);
        m_popupMenu.setPreferredSize(_preferredPopupMenuSize());
        m_popupMenu.setVisible(b);
    }

    private Dimension _preferredPopupMenuSize() {
        // Based on some heuristics...
        int n = Math.min(m_comboBox.getItemCount(), 8);
        int height = (int)(18.4 * n) + 2;
        int width = m_popupMenu.getPreferredSize().width;
        return new Dimension(width, height);
    }
    
    public class ComboBoxItem {
        private JMenuItem m_item;
        public ComboBoxItem(JMenuItem item) {
            m_item = item;
        }
        public JMenuItem getItem() {
            return m_item;
        }
        public String toString() {
            return m_item.getText();
        }
    }
}
